@@ -84,9 +84,9 @@ my %EMPTY = map { $_ => 1 } (
 my @PHRASING = (
   qw(a abbr area audio b bdi bdo br button canvas cite code data datalist),
   qw(del dfn em embed i iframe img input ins kbd keygen label link map mark),
-  qw(math meta meter noscript object output progress q ruby s samp script),
-  qw(select small span strong sub sup svg template textarea time u var video),
-  qw(wbr)
+  qw(math meta meter noscript object output picture progress q ruby s samp),
+  qw(script select small span strong sub sup svg template textarea time u),
+  qw(var video wbr)
 );
 my @OBSOLETE = qw(acronym applet basefont big font strike tt);
 my %PHRASING = map { $_ => 1 } @OBSOLETE, @PHRASING;
